@@ -1,38 +1,47 @@
-#include "Viewpoint.hpp"
+#include "Viewpoint.h"
 
-#include "opengl.hpp"
+#include "subscene.h"
+#include "opengl.h"
+#include "R.h"
 
 using namespace rgl;
 
 //////////////////////////////////////////////////////////////////////////////
 //
 // CLASS
-//   Viewpoint
+//   UserViewpoint -- the viewer's portion of the viewpoint
+//   ModelViewpoint -- the model's portion
+//
+//   These were previously just one Viewpoint class, but subscenes mean they need to be split
 //
 
-Viewpoint::Viewpoint(PolarCoord in_position, float in_fov, float in_zoom, Vec3 in_scale, bool in_interactive) :
-    SceneNode(VIEWPOINT),
+UserViewpoint::UserViewpoint(float in_fov, float in_zoom) :
+    SceneNode(USERVIEWPOINT),
     fov(in_fov),
     zoom(in_zoom),
+    viewerInScene(false)
+{
+}
+
+ModelViewpoint::ModelViewpoint(PolarCoord in_position, Vec3 in_scale, bool in_interactive) :
+    SceneNode(MODELVIEWPOINT),
     interactive(in_interactive)
 {
     scale = in_scale;
+    scaleChanged = true;    
      
     setPosition(in_position);
     clearMouseMatrix();
 }
 
-
-PolarCoord& Viewpoint::getPosition()
+PolarCoord& ModelViewpoint::getPosition()
 {
   return position;
 }
 
-Viewpoint::Viewpoint(double* in_userMatrix, float in_fov, float in_zoom, Vec3 in_scale, bool in_interactive) :
-    SceneNode(VIEWPOINT),
+ModelViewpoint::ModelViewpoint(double* in_userMatrix, Vec3 in_scale, bool in_interactive) :
+    SceneNode(MODELVIEWPOINT),
     position( PolarCoord(0.0f, 0.0f) ),
-    fov(in_fov),
-    zoom(in_zoom),
     interactive(in_interactive)
 {
     for (int i=0; i<16; i++) {
@@ -44,7 +53,8 @@ Viewpoint::Viewpoint(double* in_userMatrix, float in_fov, float in_zoom, Vec3 in
     clearMouseMatrix();
 }
 
-void Viewpoint::setPosition(const PolarCoord& in_position)
+
+void ModelViewpoint::setPosition(const PolarCoord& in_position)
 {
     Matrix4x4 M,N;
     M.setRotate(0, in_position.phi);
@@ -54,41 +64,59 @@ void Viewpoint::setPosition(const PolarCoord& in_position)
     position = in_position;
 }
 
-void Viewpoint::clearMouseMatrix()
+void ModelViewpoint::clearMouseMatrix()
 {
     Matrix4x4 M;
     M.setIdentity();
     M.getData((double*)mouseMatrix);
 }
 
-float Viewpoint::getZoom() const
+float UserViewpoint::getZoom() const
 {
   return zoom;
 }
 
-void Viewpoint::setZoom(const float in_zoom)
+void UserViewpoint::setZoom(const float in_zoom)
 {
   zoom = in_zoom;
 }
 
-bool Viewpoint::isInteractive() const
+bool ModelViewpoint::isInteractive() const
 {
   return interactive;
 }
 
-void Viewpoint::setFOV(const float in_fov)
+void UserViewpoint::setFOV(const float in_fov)
 {
   fov = clamp(in_fov, 0.0, 179.0 );
 }
 
-float Viewpoint::getFOV() const
+float UserViewpoint::getFOV() const
 {
   return fov;
 }
 
-void Viewpoint::setupFrustum(RenderContext* rctx, const Sphere& viewSphere)
+void UserViewpoint::setupFrustum(RenderContext* rctx, const Sphere& viewSphere)
 {
-  frustum.enclose(viewSphere.radius, fov, rctx->rect.width, rctx->rect.height);
+  frustum.enclose(viewSphere.radius, fov, rctx->subscene->pviewport[2], rctx->subscene->pviewport[3]);
+  if (!viewerInScene) {
+    eye.x = 0.;
+    eye.y = 0.;
+    eye.z = frustum.distance;
+  } else {
+    float oldnear = frustum.znear;
+    frustum.znear -= -eye.z + frustum.distance;
+    frustum.zfar  -= -eye.z + frustum.distance;
+    if (frustum.zfar < 0) 
+      frustum.zfar = 1;	
+    if (frustum.znear < frustum.zfar/100.)  // we lose log2(100) bits of depth resolution
+      frustum.znear = frustum.zfar/100.;
+    float ratio = frustum.znear/oldnear;
+    frustum.left   = frustum.left*ratio   + eye.x;
+    frustum.right  = frustum.right*ratio  + eye.x;
+    frustum.top    = frustum.top*ratio    + eye.y;
+    frustum.bottom = frustum.bottom*ratio + eye.y;
+  }    
 
   // zoom
 
@@ -96,40 +124,48 @@ void Viewpoint::setupFrustum(RenderContext* rctx, const Sphere& viewSphere)
   frustum.right *= zoom;
   frustum.bottom *= zoom;
   frustum.top *= zoom;
-}
-
-void Viewpoint::setupOrientation(RenderContext* rctx) const
-{
-  glMultMatrixd(mouseMatrix);
-  glMultMatrixd(userMatrix);
-
-}
-
-void Viewpoint::setupTransformation(RenderContext* rctx, const Sphere& viewSphere)
-{     
-  // projection
-
-  glMatrixMode(GL_PROJECTION);
-  glLoadIdentity();
+  
   if (frustum.ortho) {
     glOrtho(frustum.left, frustum.right, frustum.bottom, frustum.top, frustum.znear, frustum.zfar);  
   } else {
     glFrustum(frustum.left, frustum.right, frustum.bottom, frustum.top, frustum.znear, frustum.zfar);  
   }
 
-  // modelview
-
-  glMatrixMode(GL_MODELVIEW);
-  glLoadIdentity();
-
-  glTranslatef( 0.0f, 0.0f, -frustum.distance );
-
-  setupOrientation(rctx);
-  glScaled(scale.x, scale.y, scale.z);
-  glTranslatef( -viewSphere.center.x, -viewSphere.center.y, -viewSphere.center.z );
 }
 
-void Viewpoint::updateMouseMatrix(Vec3 dragStart, Vec3 dragCurrent)
+Vertex UserViewpoint::getObserver()
+ {
+  return this->eye;
+}
+
+void UserViewpoint::setObserver(bool automatic, Vertex eye)
+ {
+  viewerInScene = !automatic;
+  if (viewerInScene && !ISNAN(eye.x) &&!ISNAN(eye.y) && !ISNAN(eye.z))
+    this->eye = eye;
+}
+
+void UserViewpoint::setupViewer()
+{
+  glTranslatef(-eye.x, -eye.y, -eye.z);
+}
+
+void ModelViewpoint::setupOrientation() const
+{
+  glMultMatrixd(mouseMatrix);
+  glMultMatrixd(userMatrix);
+
+}
+
+void ModelViewpoint::setupTransformation(Vertex center)
+{     
+  // modelview
+  setupOrientation();
+  glScaled(scale.x, scale.y, scale.z);
+  glTranslatef( -center.x, -center.y, -center.z );
+}
+
+void ModelViewpoint::updateMouseMatrix(Vec3 dragStart, Vec3 dragCurrent)
 {
 	Vec3 axis = dragStart.cross(dragCurrent);
 
@@ -144,7 +180,7 @@ void Viewpoint::updateMouseMatrix(Vec3 dragStart, Vec3 dragCurrent)
 	glPopMatrix();
 }
 
-void Viewpoint::updateMouseMatrix(PolarCoord newpos)
+void ModelViewpoint::updateMouseMatrix(PolarCoord newpos)
 {
     Matrix4x4 M,N;
     M.setRotate(0, newpos.phi);
@@ -153,7 +189,7 @@ void Viewpoint::updateMouseMatrix(PolarCoord newpos)
     M.getData((double*)mouseMatrix);
 }
 
-void Viewpoint::mouseOneAxis(Vertex dragStart,Vertex dragCurrent,Vertex axis)
+void ModelViewpoint::mouseOneAxis(Vertex dragStart,Vertex dragCurrent,Vertex axis)
 {
     float angle = math::rad2deg(dragCurrent.x-dragStart.x);
     Matrix4x4 M((double *)userMatrix);
@@ -166,7 +202,7 @@ void Viewpoint::mouseOneAxis(Vertex dragStart,Vertex dragCurrent,Vertex axis)
     glPopMatrix();
 }
 
-void Viewpoint::mergeMouseMatrix()
+void ModelViewpoint::mergeMouseMatrix()
 {
     Matrix4x4 M((double *)userMatrix), N((double *)mouseMatrix);
     M = N * M;
@@ -175,26 +211,26 @@ void Viewpoint::mergeMouseMatrix()
     N.getData((double *)mouseMatrix);
 }
 
-void Viewpoint::getUserMatrix(double* dest)
+void ModelViewpoint::getUserMatrix(double* dest)
 {
 	for(int i=0; i<16;i++)
 		dest[i] = userMatrix[i];
 }
 
-void Viewpoint::setUserMatrix(double* src)
+void ModelViewpoint::setUserMatrix(double* src)
 {
 	for(int i=0; i<16;i++)
 		userMatrix[i] = src[i];
 }
 
-void Viewpoint::getScale(double* dest)
+void ModelViewpoint::getScale(double* dest)
 {
     dest[0] = scale.x;
     dest[1] = scale.y;
     dest[2] = scale.z;
 }
 
-void Viewpoint::setScale(double* src)
+void ModelViewpoint::setScale(double* src)
 {
     scale.x = src[0];
     scale.y = src[1];
@@ -202,19 +238,14 @@ void Viewpoint::setScale(double* src)
     scaleChanged = true;
 }
 
-void Viewpoint::getPosition(double* dest)
+void ModelViewpoint::getPosition(double* dest)
 {
     dest[0] = position.theta;
     dest[1] = position.phi;
 }
 
-void Viewpoint::setPosition(double* src)
+void ModelViewpoint::setPosition(double* src)
 {
     position.theta = src[0];
     position.phi = src[1];
 }
-
-Vertex Viewpoint::getCOP(const Sphere& viewSphere) const
-{
-  return viewSphere.center + ( position.vector() * frustum.distance * 2.0f );
-}
