@@ -1,10 +1,10 @@
 // C++ source
 // This file is part of RGL.
 //
-// $Id: gui.cpp 976 2013-10-04 15:06:19Z murdoch $
+// $Id: gui.cpp 1115 2014-07-18 13:51:22Z murdoch $
 // ---------------------------------------------------------------------------
-#include "gui.hpp"
-#include "lib.hpp"
+#include "gui.h"
+#include "lib.h"
 
 using namespace rgl;
 
